var TeslaCloud = window.TeslaCloud || {};
TeslaCloud.Pluralization = window.TeslaCloud.Pluralization || {};

!function ($, window, document, _undefined)
{
    "use strict";

    TeslaCloud.Time = {
        ALL: 0x3F,
        SECONDS: 0x1,
        MINUTES: 0x2,
        HOURS: 0x4,
        DAYS: 0x8,
        MONTHS: 0x10,
        YEARS: 0x20,

        NO_ZEROS: 0x800,
        NO_FIRST_ZEROS: 0x1000,

        FORMAT_STR: '{years_phrased} {months_phrased} {days_phrased} {hours_phrased} {minutes_phrased} {seconds_phrased}',

        timeToObject: function (time, flags = this.ALL | this.NO_FIRST_ZEROS)
        {
            const results = {};

            if (flags & this.YEARS)
            {
                const years = Math.floor(time / 31536000);
                if (years || !(flags & (this.NO_ZEROS | this.NO_FIRST_ZEROS)))
                {
                    results['years'] = years;
                    time %= 31536000;
                }
            }

            if (flags & this.MONTHS)
            {
                const months = Math.floor(time / 2592000);
                if (months || !((flags & this.NO_ZEROS) || !(flags & this.NO_FIRST_ZEROS && !$.isEmptyObject(results))))
                {
                    results['months'] = months;
                    time %= 2592000;
                }
            }

            if (flags & this.DAYS)
            {
                const days = Math.floor(time / 86400);
                if (days || !((flags & this.NO_ZEROS) || !(flags & this.NO_FIRST_ZEROS && !$.isEmptyObject(results))))
                {
                    results['days'] = days;
                    time %= 86400;
                }
            }

            if (flags & this.HOURS)
            {
                const hours = Math.floor(time / 3600);
                if (hours || !((flags & this.NO_ZEROS) || !(flags & this.NO_FIRST_ZEROS && !$.isEmptyObject(results))))
                {
                    results['hours'] = hours;
                    time %= 3600;
                }
            }

            if (flags & this.MINUTES)
            {
                const minutes = Math.floor(time / 60);
                if (minutes || !((flags & this.NO_ZEROS) || !(flags & this.NO_FIRST_ZEROS && !$.isEmptyObject(results))))
                {
                    results['minutes'] = minutes;
                    time %= 60;
                }
            }

            if (flags & this.SECONDS)
            {
                const seconds = Math.floor(time);
                if (seconds || !((flags & this.NO_ZEROS) || !(flags & this.NO_FIRST_ZEROS && !$.isEmptyObject(results))))
                {
                    results['seconds'] = seconds;
                }
            }

            return results;
        },

        timeToString: function (
            time,
            format = this.FORMAT_STR,
            flags = this.ALL | this.NO_FIRST_ZEROS
        )
        {
            const params = {
                    years: 0,
                    years_phrased: '',
                    months: 0,
                    months_phrased: '',
                    days: 0,
                    days_phrased: '',
                    hours: 0,
                    hours_phrased: '',
                    minutes: 0,
                    minutes_phrased: '',
                    seconds: 0,
                    seconds_phrased: ''
                },
                timeObject = this.timeToObject(time, flags);

            if (timeObject.hasOwnProperty('years'))
            {
                params.years = timeObject.years;
                params.years_phrased = XF.phrase(
                    'tc_clib_time_x_years',
                    {
                        '{years}': timeObject.years
                    },
                    XF.phrase('tc_clib_time_x_years_fallback', {
                        '{years}': timeObject.years
                    })
                );
            }

            if (timeObject.hasOwnProperty('months'))
            {
                params.months = timeObject.months;
                params.months_phrased = XF.phrase(
                    'tc_clib_time_x_months',
                    {
                        '{months}': timeObject.months
                    },
                    XF.phrase('tc_clib_time_x_months_fallback', {
                        '{months}': timeObject.months
                    })
                );
            }

            if (timeObject.hasOwnProperty('days'))
            {
                params.days = timeObject.days;
                params.days_phrased = XF.phrase(
                    'tc_clib_time_x_days',
                    {
                        '{days}': timeObject.days
                    },
                    XF.phrase('tc_clib_time_x_days_fallback', {
                        '{days}': timeObject.days
                    })
                );
            }

            if (timeObject.hasOwnProperty('hours'))
            {
                params.hours = timeObject.hours;
                params.hours_phrased = XF.phrase(
                    'tc_clib_time_x_hours',
                    {
                        '{hours}': timeObject.hours
                    },
                    XF.phrase('tc_clib_time_x_hours_fallback', {
                        '{hours}': timeObject.hours
                    })
                );
            }

            if (timeObject.hasOwnProperty('minutes'))
            {
                params.minutes = timeObject.minutes;
                params.minutes_phrased = XF.phrase(
                    'tc_clib_time_x_minutes',
                    {
                        '{minutes}': timeObject.minutes
                    },
                    XF.phrase('tc_clib_time_x_minutes_fallback', {
                        '{minutes}': timeObject.minutes
                    })
                );
            }

            if (timeObject.hasOwnProperty('seconds'))
            {
                params.seconds = timeObject.seconds;
                params.seconds_phrased = XF.phrase(
                    'tc_clib_time_x_seconds',
                    {
                        '{seconds}': timeObject.seconds
                    },
                    XF.phrase('tc_clib_time_x_seconds_fallback', {
                        '{seconds}': timeObject.seconds
                    })
                );
            }

            return $.trim(
                format.replaceAll(/\{([a-z0-9_-]+)}/ig, function (match, key)
                {
                    if (params.hasOwnProperty(key))
                    {
                        return params[key];
                    }

                    return match;
                }).replaceAll(/\s+/g, ' ')
            );
        }
    };
}
(jQuery, window, document);